@@ -1,13 +1,14 @@
 #![warn(missing_docs)]
+#![warn(missing_doc_code_examples)]
 
 //! A crate for dealing with the Gemini (and Mercury) protocol
 //!
 //! This crate is split into three parts:
-//! - gemtext
-//! - req_resp
-//! - url
+//! - [`gemtext`]
+//! - [`req_resp`]
+//! - [`url`]
 //!
-//! The gemtext module is used purely for the parsing of gemtext. The req_resp
+//! The [`gemtext`] module is used purely for the parsing of gemtext. The [`req_resp`]
 //! module is used for making requests to gemini/mercury servers and parsing them. The
 //! url module is used for manipulating URLs and only contains the areas of the URL specification
 //! that Gemini requires.
