@@ -69,7 +69,21 @@ pub enum UrlParseError {
 }
 
 impl Url {
-    /// Parse a URL from a &str object.
+    /// Parse a URL from a [`&str`] object.
+    ///
+    /// # Example:
+    /// ```
+    /// # use gmi::url::Url;
+    /// # use gmi::url::UrlParseError;
+    /// # fn main() -> Result<(), UrlParseError> {
+    /// let url = Url::from_str("gemini://example.com")?;
+    /// assert_eq!(url.scheme, Some(String::from("gemini")));
+    /// assert_eq!(url.authority.host, "example.com");
+    /// assert_eq!(url.path, None);
+    /// assert_eq!(url.query, None);
+    /// # Ok(())
+    /// # }
+    /// ```
     pub fn from_str(raw_url: &str) -> Result<Self, UrlParseError> {
         if raw_url.trim().is_empty() {
             return Err(UrlParseError::EmptyURL);
@@ -179,10 +193,20 @@ pub enum AuthorityParseError {
 impl Authority {
     /// Parse a str into an Authority.
     ///
-    /// Returns: A complete [Authority structure](struct.Authority.html)
+    /// # Returns:
+    /// A complete [`Authority structure`](Authority)
     ///
-    /// Error:
-    /// See [AuthorityParseError](enum.AuthorityParseError.html)
+    /// # Error:
+    /// See [`AuthorityParseError`])
+    /// # Example
+    /// ```
+    /// # use gmi::url::Authority;
+    /// # fn main() -> Result<(), gmi::url::AuthorityParseError> {
+    /// let auth = Authority::parse_str("example.com:1963")?;
+    /// assert_eq!(auth.port, Some(1963));
+    /// assert_eq!(auth.host, "example.com");
+    /// # Ok(())
+    /// # }
     pub fn parse_str(s: &str) -> Result<Self, AuthorityParseError> {
         let host;
         let mut port = None;
@@ -214,7 +238,7 @@ impl Authority {
 ///
 /// This part is optional in a URL and specifies the specific resource to access
 ///
-/// This implementation is based on the API of std::path::Path
+/// This implementation is based on the API of [`std::path::Path`]
 #[derive(Debug, PartialEq, Eq, Clone)]
 pub struct Path {
     /// The raw path string of the URL
@@ -233,20 +257,16 @@ impl From<&str> for Path {
     }
 }
 
-/// An iterator type used by the Path struct
-pub struct Ancestors<'a> {
-    path: &'a Path,
-}
-
-impl Iterator for Ancestors<'_> {
-    type Item = Path;
-    fn next(&mut self) -> Option<Self::Item> {
-        self.path.parent()
-    }
-}
-
 impl Path {
-    /// Turns a &str to a path
+    /// Turns a [`&str`] to a [`Path`]
+    ///
+    /// # Example:
+    /// ```
+    /// # use gmi::url::Path;
+    /// # fn main() {
+    /// let path = Path::parse_str("/help/me");
+    /// assert_eq!(path.to_string(), "/help/me");
+    /// # }
     pub fn parse_str(raw_string: &str) -> Self {
         Self {
             raw_path: raw_string.to_owned(),
@@ -254,11 +274,35 @@ impl Path {
     }
 
     /// Returns a path with the parent part
+    ///
+    /// # Example
+    /// ```
+    /// # fn main() {
+    /// use gmi::url::Path;
+    /// assert_eq!(Path::from("/hi/hello/").parent(), Some(Path::from("/hi/")));
+    /// # }
+    /// ```
     pub fn parent(&self) -> Option<Self> {
-        // Just split on the last "/" and give that back
-        match self.raw_path.rsplit_once('/') {
+        // If the path is the root, there is no parent
+        if self.raw_path == "/" {
+            return None;
+        }
+        // If the path is empty, there is no parent
+        if self.raw_path == "" {
+            return None;
+        }
+        let raw_path = {
+            // Check if the path ends in a slash
+            if self.raw_path.ends_with('/') {
+                self.raw_path[..self.raw_path.len() - 1].to_owned()
+            } else {
+                self.raw_path.clone()
+            }
+        };
+        // Split on the final / and give that back
+        match raw_path.rsplit_once('/') {
             None => None,
-            Some((parent, _)) => Some({
+            Some((parent,_)) => Some({
                 let mut raw_path = parent.to_owned();
                 raw_path.push('/');
                 Self { raw_path }
@@ -267,23 +311,43 @@ impl Path {
     }
 
     /// Returns true if the path is an absolute path
+    ///
+    /// # Example
+    /// ```
+    /// # use gmi::url::Path;
+    /// # fn main() {
+    /// let path = Path::parse_str("/a/good/path");
+    /// assert!(path.is_absolute());
+    /// # }
+    /// ```
     pub fn is_absolute(&self) -> bool {
         // This is actually pretty simple. A path is absolute if it begins with a slash
         self.raw_path.starts_with('/')
     }
 
     /// Returns true if the path is a relative path
+    ///
+    /// # Example
+    /// ```
+    /// # use gmi::url::Path;
+    /// # fn main() {
+    /// let path = Path::parse_str("a/relative/path");
+    /// assert!(path.is_relative());
+    /// # }
+    /// ```
     pub fn is_relative(&self) -> bool {
         !self.is_absolute()
     }
 
-    /// Returns an iterator that iterates on all the ancestors of the current
-    /// path
-    pub fn ancestors(&self) -> Ancestors<'_> {
-        Ancestors { path: self }
-    }
-
     /// Returns the file name with no associated hierarchy
+    ///
+    /// # Example
+    /// ```
+    /// # fn main() {
+    /// use gmi::url::Path;
+    /// assert_eq!(Path::from("/help/me.hi").file_name(), Some("me.hi"));
+    /// # }
+    /// ```
     pub fn file_name(&self) -> Option<&str> {
         // Is the path empty?
         if self.raw_path.trim().is_empty() {
@@ -297,7 +361,15 @@ impl Path {
         return Some(self.raw_path.trim().rsplit_once('/').unwrap().1);
     }
 
-    /// Merges two paths
+    /// Creates a new [`Path`] with a different [`Path`] adjoined to `self`.
+    ///
+    /// # Example
+    /// ```
+    /// # fn main() {
+    /// use gmi::url::Path;
+    /// assert_eq!(Path::from("/help/").merge_path(&Path::from("me")), Path::from("/help/me"));
+    /// # }
+    /// ```
     pub fn merge_path(&self, other_path: &Self) -> Self {
         // If the other path is empty, we don't need to do anything
         if other_path.raw_path.trim().is_empty() {
@@ -311,7 +383,7 @@ impl Path {
             }
         } else {
             // The other path is relative, so we'll just append the other path to this path.
-            // If this path is empty, we can just set the path to the other path, but relative.
+            // If this path is empty, we can just set the path to the other path, but absolute.
             if self.raw_path.trim().is_empty() {
                 let mut new_path = String::from("/");
                 new_path.push_str(&other_path.raw_path);
@@ -339,6 +411,15 @@ impl Path {
     }
 
     /// Removes any relative dot pathing from the path.
+    ///
+    /// # Example
+    /// ```
+    /// # use gmi::url::Path;
+    /// # fn main() {
+    /// let mut path = Path::parse_str("/a/dotted/../path/./with/stuff");
+    /// path.dedotify();
+    /// assert_eq!(path.to_string(), "/a/path/with/stuff");
+    /// # }
     pub fn dedotify(&mut self) {
         // Input buffer
         let mut input = self.raw_path.clone();
@@ -414,7 +495,7 @@ impl Path {
 /// This part is optional in a URL and consists of "fragments"
 /// The first query part is the first fragment, and each fragment following is separated by the
 /// character '#'
-#[derive(Debug)]
+#[derive(Debug, Eq, PartialEq)]
 pub struct Query {
     /// The various fragments of the query portion of the URL
     pub fragments: Vec<String>,
@@ -433,7 +514,7 @@ impl std::fmt::Display for Query {
 impl From<&str> for Query {
     /// # NOTE
     ///
-    /// See [parse_str](struct.Query.html#method.parse_str) for various
+    /// See [`parse_str`](Query::parse_str) for various
     /// parsing infos
     fn from(s: &str) -> Self {
         Self::parse_str(s)
@@ -445,6 +526,15 @@ impl Query {
     ///
     /// This requires the query part to be already separated from the URL. The str will not start
     /// with a '?', and if it does, the first fragment will contain it
+    ///
+    /// # Examples:
+    /// ```
+    /// # use gmi::url::Query;
+    /// # fn main() {
+    /// let query = Query::parse_str("test#query");
+    /// assert_eq!(query.fragments[0], "test");
+    /// assert_eq!(query.fragments[1], "query");
+    /// # }
     pub fn parse_str(raw_query: &str) -> Self {
         Self {
             fragments: raw_query.split('#').map(|s| String::from(s)).collect(),
@@ -455,7 +545,17 @@ impl Query {
 /// Returns if a specific character is part of the reserved
 /// characters list of the URL
 ///
-/// See section
+/// See [section 2.2 of RFC 3986](https://datatracker.ietf.org/doc/html/rfc3986#section-2.2) for
+/// more details
+///
+/// # Example
+/// ```
+/// # use gmi::url;
+/// # fn main() {
+/// assert!(url::is_reserved_char('!'));
+/// assert!(!url::is_reserved_char('c'));
+/// # }
+/// ```
 pub fn is_reserved_char(c: char) -> bool {
     // All alphanumeric characters are unreserved
     if c.is_alphanumeric() {
@@ -468,6 +568,18 @@ pub fn is_reserved_char(c: char) -> bool {
 }
 
 /// Percent encodes any reserved characters in a string
+///
+/// See [section 2.1 of RFC 3986](https://datatracker.ietf.org/doc/html/rfc3986#section-2.1) for
+/// more details
+///
+/// # Example
+/// ```
+/// # use gmi::url;
+/// # fn main() {
+/// let percent_encoding = url::percent_encode_reserved_characters("this is a test");
+/// assert_eq!(percent_encoding, "this%20is%20a%20test");
+/// # }
+/// ```
 pub fn percent_encode_reserved_characters(data: &str) -> String {
     let mut ret = String::new();
     for c in data.chars() {
@@ -481,6 +593,14 @@ pub fn percent_encode_reserved_characters(data: &str) -> String {
 }
 
 /// Percent encodes a singular character
+/// See [`percent_encode_reserved_characters`] for more info on this
+///
+/// # Example:
+/// ```
+/// # use gmi::url;
+/// # fn main() {
+/// assert_eq!(url::percent_encode('!'), "%21");
+/// # }
 pub fn percent_encode(c: char) -> String {
     let mut ret = String::new();
     let mut buf = [0; 4];
